<?php
/*
Plugin Name: 5sec Snow
Plugin URI: http://5sec-snow.webfactoryltd.com/
Description: Beautiful, easily configurable snow for any website in just a few clicks.
Author: 20script
Version: 1.60
Author URI: http://www.20script.ir/
*/


if (!function_exists('add_action')) {
  die('Please don\'t open this file directly!');
}


define('WF_SNOW_CORE_VER', 1.60);
define('WF_SNOW_OPTIONS_KEY', 'wf_snow');


class wf_snow {
  static $version = 1.60;


  // add hooks and filters
  static function init() {
    if (is_admin()) {
      // check if minimal required WP version is used
      self::check_wp_version(4.1);

      // enqueue scripts
      add_action('admin_enqueue_scripts', array(__CLASS__, 'enqueue_scripts'));

      // aditional links in plugin description
      add_filter('plugin_action_links_' . basename(dirname(__FILE__)) . '/' . basename(__FILE__),
                 array(__CLASS__, 'plugin_action_links'));
      add_filter('plugin_row_meta', array(__CLASS__, 'plugin_meta_links'), 10, 2);

      // add menu item
      add_action('admin_menu', array(__CLASS__, 'admin_menu'));

      // if the plugin was installed/reinstalled/broken set settings to default
      self::default_settings(false);

      // settings registration
      add_action('admin_init', array(__CLASS__, 'register_settings'));

      // manage admin header notifications
      add_action('admin_notices', array(__CLASS__, 'notices'));
    } else {
      // add JS to frontend
      add_action('wp_enqueue_scripts', array(__CLASS__, 'frontend_enqueue_scripts'));
      add_action('wp_footer', array(__CLASS__, 'frontend_footer'));
    }
  } // init


  // all settings are saved in one option key
  static function register_settings() {
    register_setting(WF_SNOW_OPTIONS_KEY, 'wf_snow', array(__CLASS__, 'sanitize_settings'));
  } // register_settings


  // display notice if Snow is enabled
  static function notices() {
    $options = get_option(WF_SNOW_OPTIONS_KEY);
    $screen = get_current_screen();

    if ($options['status'] && !$options['admin_notice'] && $screen->id != 'appearance_page_wf-snow') {
      $disabled = false;
      if ($options['active-until']) {
        $last_day = strtotime($options['active-until'] . ' midnight');
        if (current_time('timestamp') > $last_day) {
          $disabled = true;
        }
      }
      if ($disabled) {
        echo '<div id="wf-snow-enabled" class="updated"><p><b>5sec Snow</b> is enabled but it\'s no longer active. It was active until ' . $options['active-until'] . ' 00:00. Manage <a href="themes.php?page=wf-snow">settings</a>.</p></div>';
      } else {
        echo '<div id="wf-snow-enabled" class="updated"><p><b>5sec Snow</b> is enabled. Manage <a href="themes.php?page=wf-snow">settings</a>.</p></div>';
      }
    } // if enabled
  } // notice


  // set default settings
  static function default_settings($force = false) {
    $defaults = array('status' => 0,
                      'flakes' => 'big-24',
                      'nb_flakes' => 30,
                      'speed' => 5,
                      'wind' => 30,
                      'melting' => 600,
                      'rotation' => 6,
                      'active-until' => '2019-01-06',
                      'turn_off_link' => 1,
                      'turn_off_html' => '<div id="wf_snow_control" style="position: fixed; top: 0; left: 0; display: inline; padding: 10px; margin: 20px; background: #bababa; border:1px solid #aaa; opacity: 0.8; border-radius: 5px; -moz-border-radius: 5px; -webkit-border-radius:5px; box-shadow: 1px 1px 3px #000000; -moz-box-shadow: 1px 1px 3px #000000; -webkit-box-shadow: 1px 1px 3px #222"><a href="{LINK}" style="color: #fff">Turn off snow</a></div>',
                      'linked_flakes' => 0,
                      'link' => '',
                      'only_frontpage' => 0,
                      'small_screen' => 0,
                      'include_jquery' => 1,
                      'use_minified_js' => 0,
                      'admin_notice' => 0);

    $options = get_option(WF_SNOW_OPTIONS_KEY);

    if ($force || !$options || !$options['flakes']) {
      update_option(WF_SNOW_OPTIONS_KEY, $defaults);
    }
  } // default_settings


  // sanitize settings on save
  static function sanitize_settings($values) {
    $old_options = get_option(WF_SNOW_OPTIONS_KEY);
    set_transient('wf-snow-updated', true, 10);

    foreach ($values as $key => $value) {
      switch ($key) {
        case 'status':
        case 'nb_flakes':
        case 'speed':
        case 'wind':
        case 'melting':
        case 'rotation':
        case 'linked_flakes':
        case 'turn_off_link':
        case 'admin_notice':
        case 'only_frontpage':
        case 'include_jquery':
        case 'use_minified_js':
        case 'small_screen':
          $values[$key] = (int) $value;
        break;
        case 'flakes':
        case 'link':
        case 'active-until':
        case 'turn_off_html':
          $values[$key] = trim($value);
        break;
      } // switch
    } // foreach

    self::check_var_isset($values, array('turn_off_link' => 0, 'admin_notice' => 0, 'include_jquery' => 0, 'only_frontpage' => 0, 'use_minified_js' => 0));

    return array_merge($old_options, $values);
  } // sanitize_settings


  // create the admin menu item under appearance
  static function admin_menu() {
    add_theme_page('5sec Snow', '5sec Snow', 'edit_theme_options', 'wf-snow', array(__CLASS__, 'options_page'));
  } // admin_menu


  // check if user has the minimal WP version required by the plugin
  static function check_wp_version($min_version) {
    if (!version_compare(get_bloginfo('version'), $min_version,  '>=')) {
        add_action('admin_notices', array(__CLASS__, 'min_version_error'));
    }
  } // check_wp_version


  // display error message if WP version is too low
  static function min_version_error() {
    echo '<div class="error"><p>5sec Snow <b>requires WordPress version 4.1</b> or higher to function properly. You\'re using WordPress version ' . get_bloginfo('version') . '. Please <a href="update-core.php">update it</a>.</p></div>';
  } // min_version_error


  // enqueue CSS and JS scripts for snow admin
  static function enqueue_scripts() {
    $screen = get_current_screen();

    if ($screen->id == 'appearance_page_wf-snow') {
      $plugin_url = plugin_dir_url(__FILE__);

      wp_enqueue_style('wf-snow-be', $plugin_url . 'css/5sec-snow-backend.css', array(), WF_SNOW_CORE_VER);
      wp_enqueue_script('wf-snow-be', $plugin_url . 'js/5sec-snow-backend.js', array('jquery'), WF_SNOW_CORE_VER);
      wp_enqueue_script('wf-ccss-anytime', $plugin_url . 'js/5sec-snow-anytimec.js', array('jquery'), WF_SNOW_CORE_VER);
    } // if snow screen
  } // enqueue_scripts


  // check if snow is enabled for current user
  static function snow_enabled() {
    $options = get_option(WF_SNOW_OPTIONS_KEY);
    $last_day = strtotime($options['active-until'] . ' midnight');

    if (!$options['status']
        || ($options['only_frontpage'] && (!is_home() && !is_front_page()))
        || ($options['active-until'] && current_time('timestamp') > $last_day)
        || isset($_COOKIE['wf_5sec_snow_disable'])) {
      return false;
    } else {
      return true;
    }
  } // snow_enabled


  // add frontend JS
  static function frontend_enqueue_scripts() {
    if (!self::snow_enabled()) {
      return;
    }

    $options = get_option(WF_SNOW_OPTIONS_KEY);
    if ($options['use_minified_js']) {
      $file = '5sec-snow.min.js';
    } else {
      $file = '5sec-snow.js';
    }

    if ($options['include_jquery']) {
      wp_enqueue_script('wf-snow', plugin_dir_url(__FILE__) . 'js/' . $file, array('jquery'), WF_SNOW_CORE_VER, true);
    } else {
      wp_enqueue_script('wf-snow', plugin_dir_url(__FILE__) . 'js/' . $file, array(), WF_SNOW_CORE_VER, true);
    }
  } // frontend_enqueue_scripts


  // print JS in footer
  static function frontend_footer() {
    if (!self::snow_enabled()) {
      return;
    }

    $options = get_option(WF_SNOW_OPTIONS_KEY);
    $plugin_url = plugin_dir_url(__FILE__);


    if ($options['flakes'] == 'html') {
      $flakes_dir = "['*', '&curren;', '&bull;', '&diams;', '+']";
    } else {
      $flakes_dir = "'" . $plugin_url . 'images/' . $options['flakes'] . "/'";
    }

    $out  = '<script type="text/javascript">';
    $out .= "jQuery(document).ready(function($){
               var snow_opt = { flake_number: {$options['nb_flakes']},
                                flake_folder: {$flakes_dir},
                                flake_imgs: 6,
                                linked_flakes: {$options['linked_flakes']},
                                link: '{$options['link']}',
                                melt: {$options['melting']},
                                wind: {$options['wind']},
                                rotation: {$options['rotation']},
                                speed: {$options['speed']}};
               if ($(window).width() >= parseInt({$options['small_screen']}, 10) && $(window).height() >= parseInt({$options['small_screen']}, 10)) {
                 $.snow(snow_opt);
               } else {
                 $('#wf_snow_control').hide();
               }
             });";
    $out .= "</script>\n";

    if ($options['turn_off_link'] && $options['turn_off_html']) {
      $out .= str_replace('{LINK}', $plugin_url . 'turn-off.php?return=' . urlencode($_SERVER['REQUEST_URI']), $options['turn_off_html']);
    }

    echo $out;
  } // frontend_footer


  // add settings link to plugins page
  static function plugin_action_links($links) {
    $settings_link = '<a href="themes.php?page=wf-snow" title="Configure 5sec Snow plugin">Configure</a>';
    array_unshift($links, $settings_link);

    return $links;
  } // plugin_action_links


  // add links to plugin's description in plugins table
  static function plugin_meta_links($links, $file) {
    $documentation_link = '<a target="_blank" href="' . plugin_dir_url(__FILE__) . 'documentation/" title="View 5sec Snow documentation">Documentation</a>';
    $support_link = '<a target="_blank" href="http://codecanyon.net/user/WebFactory#from" title="Contact Web factory">Support</a>';

    if ($file == plugin_basename(__FILE__)) {
      $links[] = $documentation_link;
      $links[] = $support_link;
    }

    return $links;
  } // plugin_meta_links


  // helper function for creating dropdowns
  static function create_select_options($options, $selected = null, $output = true) {
    $out = "\n";

    foreach ($options as $tmp) {
      if ($selected == $tmp['val']) {
        $out .= "<option selected=\"selected\" value=\"{$tmp['val']}\">{$tmp['label']}&nbsp;</option>\n";
      } else {
        $out .= "<option value=\"{$tmp['val']}\">{$tmp['label']}&nbsp;</option>\n";
      }
    } // foreach

    if ($output) {
      echo $out;
    } else {
      return $out;
    }
  } // create_select_options


  // helper function for $_POST checkbox handling
  static function check_var_isset(&$values, $variables) {
    foreach ($variables as $key => $value) {
      if (!isset($values[$key])) {
        $values[$key] = $value;
      }
    }
  } // check_var_isset


  // complete options page markup
  static function options_page() {
    if (!current_user_can('edit_theme_options'))  {
      wp_die('You do not have sufficient permissions to access this page.');
    }

    $options = get_option(WF_SNOW_OPTIONS_KEY);

    if (get_transient('wf-snow-updated')) {
      echo '<div class="updated"><p><b>Changes saved.</b></p></div>';
      delete_transient('wf-snow-updated');
    }

    echo '<div class="wrap">
          <div class="icon32-new icon-snow"><br /></div>
          <h2>5sec Snow</h2>';

    echo '<form action="options.php" method="post">';
    settings_fields('wf_snow');

    echo '<h3>General Settings</h3>';
    echo '<table class="form-table"><tbody>';

    $statuses =   array();
    $statuses[] = array('val' => '0', 'label' => 'Disabled');
    $statuses[] = array('val' => '1', 'label' => 'Enabled');

    $flakes =   array();
    $flakes[] = array('val' => 'big-24', 'label' => 'Big', 'image' => 'big.jpg');
    $flakes[] = array('val' => 'medium-10', 'label' => 'Medium', 'image' => 'medium.jpg');
    $flakes[] = array('val' => 'html', 'label' => 'HTML', 'image' => 'html.jpg');
    $flakes[] = array('val' => 'small-02', 'label' => 'Small', 'image' => 'small.jpg');
    $flakes[] = array('val' => 'small-dirty-02', 'label' => 'Small dirty', 'image' => 'small-dirty.jpg');
    $flakes[] = array('val' => 'ice-24', 'label' => 'Ice', 'image' => 'ice.jpg');
    $flakes[] = array('val' => 'presents-32', 'label' => 'Presents', 'image' => 'presents.jpg');
    $flakes[] = array('val' => 'ornaments-16', 'label' => 'Ornaments', 'image' => 'ornaments.jpg');

    $nb_flakes = array();
    for ($i = 5; $i<=110; $i+=5) {
      $nb_flakes[] = array('val' => $i, 'label' => $i . ' flakes&nbsp;');
    }

    $speeds =   array();
    $speeds[] = array('val' => '1', 'label' => 'Very slow');
    $speeds[] = array('val' => '3', 'label' => 'Slow');
    $speeds[] = array('val' => '5', 'label' => 'Normal');
    $speeds[] = array('val' => '8', 'label' => 'Fast');
    $speeds[] = array('val' => '13', 'label' => 'Very fast');

    $winds =   array();
    $winds[] = array('val' => '0', 'label' => 'No wind');
    $winds[] = array('val' => '30', 'label' => 'Some wind');
    $winds[] = array('val' => '60', 'label' => 'Moderate wind');
    $winds[] = array('val' => '100', 'label' => 'Strong wind');
    $winds[] = array('val' => '200', 'label' => 'Blizzard');

    $meltings =   array();
    $meltings[] = array('val' => '0', 'label' => 'No melting, flakes just fall off the screen');
    $meltings[] = array('val' => '300', 'label' => 'Fast melting');
    $meltings[] = array('val' => '600', 'label' => 'Normal melting');
    $meltings[] = array('val' => '1000', 'label' => 'Slow melting');

    $rotations =   array();
    $rotations[] = array('val' => '0', 'label' => 'Don\'t rotate the flakes');
    $rotations[] = array('val' => '4', 'label' => 'Slow rotation');
    $rotations[] = array('val' => '6', 'label' => 'Normal rotation');
    $rotations[] = array('val' => '10', 'label' => 'Fast rotation');

    $links = array();
    $links[] = array('val' => '0', 'label' => 'No flakes are links');
    for ($i = 10; $i<=90; $i+=10) {
      $links[] = array('val' => $i, 'label' => $i . '% of all flakes are links');
    }
    $links[] = array('val' => '100', 'label' => 'All flakes are links');

    echo '<tr valign="top">
    <th scope="row"><label for="status">Status</label></th>
    <td><select id="status" name="wf_snow[status]">';
      self::create_select_options($statuses, $options['status']);
    echo '</select>';
    echo '<span>If enabled all users will see the snow except those who clicked the "Turn off snow" button on your site.</span>';
    echo '</td></tr>';

    $img = plugins_url('/images/', __FILE__);
    echo '<tr valign="top">
    <th scope="row">Flakes type</th>
    <td>';

    $i = 1;
    foreach ($flakes as $flake) {
      echo '<div class="snow-thumb"><input ' . checked($flake['val'], $options['flakes'], false) . ' type="radio" id="layout-' . $i . '" name="wf_snow[flakes]" value="' . $flake['val'] . '" /> <label for="layout-' . $i . '"><img src="' . $img . $flake['image'] . '" alt="' . $flake['label'] . '" title="' . $flake['label'] . '" /></label></div>';
      $i++;
    } // foreach flake

    echo '<br style="clear: both;" />
    </td></tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="nb_flakes">Number of flakes</label></th>
    <td><select id="nb_flakes" name="wf_snow[nb_flakes]">';
      self::create_select_options($nb_flakes, $options['nb_flakes']);
    echo '</select>';
    echo '<span>Maximum number of flakes on the screen. Default: 30.</span>';
    echo '</td></tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="speed">Speed</label></th>
    <td><select id="speed" name="wf_snow[speed]">';
      self::create_select_options($speeds, $options['speed']);
    echo '</select>';
    echo '<span>Maximum flake falling speed. Default: normal.</span>';
    echo '</td></tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="wind">Wind</label></th>
    <td><select id="wind" name="wf_snow[wind]">';
      self::create_select_options($winds, $options['wind']);
    echo '</select>';
    echo '<span>Amount of wind that pushes flakes left and right. Default: some wind.</span>';
    echo '</td></tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="melting">Melting</label></th>
    <td><select id="melting" name="wf_snow[melting]">';
      self::create_select_options($meltings, $options['melting']);
    echo '</select>';
    echo '<span>If enabled flakes melt on the bottom of the screen. Default: normal melting.</span>';
    echo '</td></tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="rotation">Rotation</label></th>
    <td><select id="rotation" name="wf_snow[rotation]">';
      self::create_select_options($rotations, $options['rotation']);
    echo '</select>';
    echo '<span>If enabled flakes will rotate while falling. Default: normal rotation.</span>';
    echo '</td></tr>';

    echo '</tbody></table>';
    echo '<p class="submit"><input type="submit" value="Save Changes" class="button-primary" name="Submit" /></p>';

    echo '<h3>Advanced Settings</h3>';
    echo '<table class="form-table"><tbody>';

    echo '<tr valign="top">
    <th scope="row"><label for="active-until">Show snow until</label></th>
    <td><input class="medium-text" id="active-until" type="text" name="wf_snow[active-until]" value="' . $options['active-until'] . '" />  <input id="active_forever" type="button" class="button-secondary" value="Remove end date" />
    <span>Snow will stop displaying on midnight on the selected date.</span></td>
    </tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="linked_flakes">Flakes are links</label></th>
    <td><select id="linked_flakes" name="wf_snow[linked_flakes]">';
      self::create_select_options($links, $options['linked_flakes']);
    echo '</select>';
    echo '<span>If enabled flakes are clickable. Default: flakes are not links.</span>';
    echo '<div id="link-container"><br /><label for="link">Link:</label> <input id="link" type="text" class="regular-text" name="wf_snow[link]" value="' . htmlspecialchars($options['link']) . '" /></div>';
    echo '</td></tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="turn_off_link">Display "turn off snow" button</label></th>
    <td><input name="wf_snow[turn_off_link]" id="turn_off_link" ' . checked('1', $options['turn_off_link'], false) . ' value="1" type="checkbox" />
    <span>Displays a button on every site\'s page that enables users to turn off snow. Cookie based. Default: checked.</span>';
    echo '<div id="off_html"><textarea id="turn_off_html" rows="3" class="large-text" name="wf_snow[turn_off_html]">' . htmlspecialchars($options['turn_off_html']) . '</textarea><br />
    <span>Enter the HTML you want for the "turn off" button/link and for the href put <i>{LINK}</i>.</span></div>';
    echo '</td></tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="only_frontpage">Snow only on front page</label></th>
    <td><input name="wf_snow[only_frontpage]" id="only_frontpage" ' . checked('1', $options['only_frontpage'], false) . ' value="1" type="checkbox" />
    <span>If enabled snow will be visible only on site\'s front page. Default: unchecked.</span></td>
    </tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="small_screen">Disable on small screens</label></th>
    <td><input class="small-text" id="small_screen" type="text" name="wf_snow[small_screen]" value="' . $options['small_screen'] . '" /> px
    <span>If screen width or height is less than the set value snow will be hidden. Set to zero to show on all screen sizes. Default: 0, recommended 640px.</span></td>
    </tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="admin_notice">Disable admin notice</label></th>
    <td><input name="wf_snow[admin_notice]" id="admin_notice" ' . checked('1', $options['admin_notice'], false) . ' value="1" type="checkbox" />
    <span>Disable admin header notice visible when 5sec Snow is enabled. Default: unchecked.</span></td>
    </tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="include_jquery">Include jQuery</label></th>
    <td><input name="wf_snow[include_jquery]" id="include_jquery" ' . checked('1', $options['include_jquery'], false) . ' value="1" type="checkbox" />
    <span>Disable this option only if your theme has jQuery hardcoded (which it shouldn\'t have!) and you have problems because jQuery gets included twice. Default: checked.</span></td>
    </tr>';

    echo '<tr valign="top">
    <th scope="row"><label for="use_minified_js">Use minified version of snow JS</label></th>
    <td><input name="wf_snow[use_minified_js]" id="use_minified_js" ' . checked('1', $options['use_minified_js'], false) . ' value="1" type="checkbox" />
    <span>If you\'re not using any site accelerators, minifiers or similar plugins then you can use the minified version. Difference is 4kB. Default: unchecked.</span></td>
    </tr>';

    echo '</tbody></table>';
    echo '<p class="submit"><input type="submit" value="Save Changes" class="button-primary" name="Submit" /></p>';

    echo '<a title="View documentation" target="_blank" href="' . plugin_dir_url(__FILE__) . 'documentation/">View documentation</a>';

    echo '</form></div>'; // main container
  } // options_page


  // clean up on deactivate
  static function uninstall() {
    delete_option(WF_SNOW_OPTIONS_KEY);
  } // deactivate
} // class wf_snow


// hook everything up
add_action('init', array('wf_snow', 'init'));
register_uninstall_hook(__FILE__, array('wf_snow', 'uninstall'));
